/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis.varianten;

import java.text.Collator;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Ruestung;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;

public class VariantenRuestungen implements Varianten {

    private final Collator collator;

    private final List< String > eingefuegte;

    VariantenRuestungen() {
         collator = Collator.getInstance();

         eingefuegte = new ArrayList< String >();
    }

    public void einfuegen( final String variante ) {

        final int index =
            Collections.binarySearch( getVarianten( null ), variante, collator);

        if ( index < 0 ) {

            eingefuegte.add( variante );
        }
    }

    public List< Object > getVarianten( final Held held ) {

        final List< Object > varianten = new ArrayList< Object >();

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final Ruestung ruestung : speicher.getRuestungen() ) {

            if ( ! ruestung.istZusatzruestung() ) {

                varianten.add( ruestung.getName() );
            }
        }

        varianten.addAll( eingefuegte );

        Collections.sort( varianten, collator );

        return varianten;
    }

    public boolean hatVarianten() {
        return true;
    }

    public boolean istFreieVariante() {
        return true;
    }

}
