/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basiswerte;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;

/**
 * Berechnet wie viele Punkte zum Startwert eines Basiswerts maximal
 * hinzugekauft werden duerfen.
 *
 * @author Bernhard Jung
 *
 */
public class MaxZukauf {

    /** Die Eigenschaft von der die maximal zukaufbaren Punkte abhaengen. */
    private final Eigenschaft eigenschaft;

    /**
     * Die Zahl durch die die Stufe der Eigenschaft geteilt wird, um die
     * maximal zukaufbaren Punkte zu ermitteln.
     */
    private final float divisor;

    /**
     * @param dieEigenschaft
     *            von der die maximal zukaufbaren Punkte abhaengen.
     * @param derDivisor
     *            durch den die Stufe der Eigenschaft geteilt wird, um die
     *            maximal zukaufbaren Punkte zu ermitteln.
     */
    MaxZukauf( final Eigenschaft dieEigenschaft, final float derDivisor ) {

        eigenschaft = dieEigenschaft;

        divisor = derDivisor;
    }

    /**
     * @param held
     *            der seinen Basiswert erhoehen will.
     * @return die maximal zum Basiswert hinzukaufbaren Punkte.
     */
    final int getMaxZukauf( final Held held ) {

        final int stufe = held.getStufe( eigenschaft );

        return Math.round( stufe / divisor );
    }
}
