/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.ungeeignete;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Macht einen Vor-/Nachteil zum unggeigneten Vor-/Nachteil.
 *
 * @author Bernhard Jung
 */
public class UngeeigneterVorNachteilStandard implements UngeeigneterVorNachteil {

    /** Der ungeeignete Vor-/Nachteil. */
    private final VorNachteil vorNachteil;

    /**
     * @param derVorNachteil der als ungeeignet gelten soll.
     */
    public UngeeigneterVorNachteilStandard(
            final VorNachteil derVorNachteil ) {

        vorNachteil = derVorNachteil;
    }

    public int compareTo(final UngeeigneterVorNachteil o) {
        return vorNachteil.compareTo( o.getVorNachteil() );
    }

    /** {@inheritDoc} */
    public VorNachteil getVorNachteil() {
        return vorNachteil;
    }

    /** @return {@code true} */
    public boolean istUngeeignetFuer( final Held held ) {
        return true;
    }

    @Override
    public String toString() {
        return vorNachteil.getName();
    }

    /** {@inheritDoc} */
    public IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_VOR_NACHTEIL );

        element.setContent( vorNachteil.getId() );

        return element;
    }
}
