/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.voraussetzungen;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.bausteine.util.Geschlecht;
import de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/** Fuer Zwerginnen gilt: MU mindestens 12. */
public class VoraussetzungZwergin extends VoraussetzungEigenschaft {

    private final static int MINDESTSTUFE = 12;

    /** @return <code>new Eigenschaft[]{ Eigenschaft.MU }</code> */
    @Override
    public Eigenschaft[] getEigenschaften() {
        return new Eigenschaft[]{ Eigenschaft.MU };
    }

    /** @return 12 */
    @Override
    public int getMindeststufe() {
        return MINDESTSTUFE;
    }

    /**
     * @return {@code true} wenn {@code held} eine Frau ist, sonst
     *      {@code false}
     */
    @Override
    public boolean hatMindeststufe( final Held held ) {
        return held.getGeschlecht() == Geschlecht.WEIBLICH;
    }

    /** {@inheritDoc} */
    public KonfliktBeschreibung pruefen( final Held held ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if ( ( held.getGeschlecht() == Geschlecht.WEIBLICH )
          && ( held.getStufe( Eigenschaft.MU ) < MINDESTSTUFE ) ) {

            beschreibung.einfuegen( I18n.i18n( "VoraussetzungZwergin" ) );
        }

        return beschreibung;
    }

    /** @return eine kurze Beschreibung der Voraussetzung. */
    @Override
    public final String toString() {
        return I18n.i18n( "VoraussetzungZwergin" );
    }

    /** {@inheritDoc} */
    public final IXMLElement toXml() {

        return new XMLElement( HeldenbausteinXmlTags.TAG_ZWERGIN );
    }

}
