/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.daten.factories;

import static de.bernhardjung.dsaprogramm.daten.xml.RuestungXmlTags.*;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;

import de.bernhardjung.dsaprogramm.basis.Ruestung;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;

public class FactoryRuestungen extends Factory< Ruestung > {

    @Override
    public final List< Ruestung > erzeugen(
            final IXMLElement wurzelElement ) {

        final List< Ruestung > ruestungen = new ArrayList< Ruestung >();

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final IXMLElement ruestungElement :
            wurzelElement.getChildren() ) {

            final String id = ruestungElement.getAttribute( ATTR_ID );

            final Ruestung ruestung = speicher.getRuestung( id );

            ruestung.setBe( getInt( ruestungElement, ATTR_BE ) );

            if ( ruestungElement.hasAttribute( ATTR_MINDEST_TORSO_RS ) ) {

                ruestung.setMindestTorsoRs(
                        getInt( ruestungElement, ATTR_MINDEST_TORSO_RS ) );
            }

            if ( ruestungElement.hasAttribute( ATTR_NAME ) ) {

                ruestung.setName(
                        ruestungElement.getAttribute( ATTR_NAME ) );
            }

            ruestung.setRs( getInt( ruestungElement, ATTR_RS ) );

            ruestung.setZusatzruestung(
                    getBoolean( ruestungElement, ATTR_ZUSATZRUESTUNG ) );

            ruestungen.add( ruestung );
        }

        return ruestungen;
    }
}
