/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.daten.factories.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_AUSSER;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_UNGEEIGNETE_VOR_NACH;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.bausteine.ungeeignete.UngeeigneterVorNachteil;
import de.bernhardjung.dsaprogramm.bausteine.ungeeignete.UngeeigneterVorNachteilAusser;
import de.bernhardjung.dsaprogramm.bausteine.ungeeignete.UngeeigneterVorNachteilStandard;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Ermittelt die ungeeigneten Vor-/Nachteile eines Bausteins aus dem
 * XML-Dokument.
 *
 * @author Bernhard Jung
 *
 */
public class FactoryUngeeigneteVorNachteileIn {

    /**
     * @param wurzelElement der XML-Datei.
     * @return eine Liste von Vor-/Nachteilen.
     */
    public final List< UngeeigneterVorNachteil > erzeugen( final IXMLElement wurzelElement ) {

        final List< UngeeigneterVorNachteil > vorNachteileListe =
            new ArrayList< UngeeigneterVorNachteil >();

        final IXMLElement vorNachteile =
            wurzelElement.getFirstChildNamed( TAG_UNGEEIGNETE_VOR_NACH );

        if ( null != vorNachteile ) {

            for ( final IXMLElement vorNachteil : vorNachteile.getChildren() ){

                final String id = vorNachteil.getContent();

                final String ausser =
                    vorNachteil.getAttribute( ATTR_AUSSER );

                final UngeeigneterVorNachteil ungeeigneter;

                if ( null != ausser ) {

                    ungeeigneter = new UngeeigneterVorNachteilAusser(
                            VorNachteil.getVorNachteil( id ),
                            VorNachteil.getVorNachteil( ausser ) );
                } else {

                    ungeeigneter = new UngeeigneterVorNachteilStandard(
                            VorNachteil.getVorNachteil( id ) );
                }

                vorNachteileListe.add( ungeeigneter );
            }
        }

        return vorNachteileListe;
    }
}
