/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.erschaffung;

import java.awt.event.ActionEvent;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.AbstractAction;
import javax.swing.Action;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.bausteine.bonus.SonderfertigkeitBonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.Talentbonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFest;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFrei;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteSonderfertigkeit;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VorNachteilBonus;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.PanelErschaffung;
import de.bernhardjung.dsaprogramm.gui.PanelGp;
import de.bernhardjung.dsaprogramm.gui.PanelRasKulProf;
import de.bernhardjung.dsaprogramm.gui.PanelSteigernErschaffung;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel.Warnen;
import de.bernhardjung.dsaprogramm.gui.bonus.PanelAuswahl;
import de.bernhardjung.dsaprogramm.gui.bonus.PanelLeitzauber;
import de.bernhardjung.dsaprogramm.gui.bonus.PanelTalentBonus;
import de.bernhardjung.dsaprogramm.gui.bonus.PanelTalentBonusFest;
import de.bernhardjung.dsaprogramm.gui.bonus.PanelTalentBonusFestEntdecker;
import de.bernhardjung.dsaprogramm.gui.bonus.PanelTalentBonusFrei;
import de.bernhardjung.dsaprogramm.gui.steigern.PanelSteigern;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.talente.Sprache;
import de.bernhardjung.dsaprogramm.talente.TalentWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;
import de.bernhardjung.dsaprogramm.zauber.Zauber;

/** Steuert den Ablauf der Erschaffung eines neuen Helden. */
public class Erschaffung {

    /** Der Name fuer die Action zum Beenden der Erschaffung. */
    private static final String BEENDEN_NAME =
        I18n.i18n( "Erschaffung.Beenden.Name" );

    /** Der Name fuer die Action zum Beenden der Erschaffung. */
    private static final String BEENDEN_MNEMONIC =
        I18n.i18n( "Erschaffung.Beenden.Mnemonic" );

    /** Der Text: 'Bitte waehlen Sie eine Zweitsprache.' */
    private static final String ZWEITSPRACHE_WAEHLEN =
        I18n.i18n( "Erschaffung.ZweitspracheWaehlen" );

    /** Der Text: 'Bitte waehlen Sie eine Sonderfertigkeit.' */
    private static final String SONDERFERTIGKEIT_WAEHLEN =
        I18n.i18n( "Erschaffung.SonderfertigkeitWaehlen" );

    /**
     * Ein Text der dem Benutzer erklaert dass er eine verbilligte
     * Sonderfertigkeit auswaehlen soll.
     */
    private static final String VERBILLIGTE_SONDERFERTIGKEIT_WAEHLEN =
        I18n.i18n( "Erschaffung.verbilligteSonderfertigkeitWaehlen" );

    /**
     * Ein Text der dem Benutzer erklaert dass er eine verbilligte
     * Sonderfertigkeit auswaehlen soll.
     */
    private static final String VORNACHTEIL_WAEHLEN =
        I18n.i18n( "Erschaffung.VorNachteilWaehlen" );

    /**
     * Die Aufschrift fuer den Knopf mit dem man zum naechsten Schritt der
     * Erschaffung kommt.
     */
    private static final String WEITER =
        I18n.i18n( "Erschaffung.Weiter.Name" );

    /**
     * Die Taste mit der man den Knopf mit dem man zum naechsten Schritt der
     * Erschaffung kommt aktiviert.
     */
    private static final String WEITER_MNEMONIC =
        I18n.i18n( "Erschaffung.Weiter.Mnemonic" );

    /** Action mit der die Erschaffung abgeschlossen wird. */
    private final ActionBeenden actionBeenden;

    /**
     * Die Action die nach dem Auswaehlen der Leitzauber ausgefuehrt wird.
     */
    private final Action actionLeitzauber;

    /**
     * Die Action mit der nacheinander die offenen Auswahlen fuer die
     * Sonderfertigkeiten angewendet werden.
     */
    private final Action actionSonderfertigkeitBoniOffen;

    /**
     * Action mit der zum 2ten Schritt der Erschaffung gegangen wird, dem
     * anwenden der Talent-Boni.
     */
    private final ActionTalentBoni actionTalentBoni;

    /**
     * Action mit der nacheinander die offenen Talent-Boni aus Rasse und
     * Kultur ausgewertet werden.
     */
    private final Action actionTalentBoniOffen;

    /**
     * Action mit der nacheinander die offenen Talent-Boni die der Held durch
     * seine Profession erhalten hat angewendet werden.
     */
    private final Action actionTalentBoniOffenProfession;

    /**
     * Die Action die verwendet wird, wenn aus einer Liste eine verbilligte
     * Sonderfertigkeit gewaehlt werden muss.
     */
    private final Action actionVerbilligteSonderfertigkeit;

    /**
     * Die Action mit der die offenen Boni auf die Vor-/Nachteile abgearbeitet
     * werden.
     */
    private final Action actionVorNachteilBoniOffen;

    /** Action die verwendet wird um die Zweitsprache des Helden zu waehlen. */
    private final Action actionZweitsprache = new ActionZweitsprache();

    /** Das Hauptfenster der Anwendung. */
    private final Hauptpanel hauptpanel;

    /** Der Manager fuer die aktuell laufende Erschaffung. */
    private ErschaffungManager manager;

    /** Das Panel fuer die Heldenerschaffung. */
    private final PanelErschaffung panelErschaffung;

    /**
     * Das Panel mit dem die zusaetzlichen Leitzauber fuer eien elfischen
     * Helden gewaehlt werden koennen.
     */
    private PanelLeitzauber panelLeitzauber;

    /**
     * Die Schnittstelle die es erlaubt eine Sonderfertigkeit fuer den Helden
     * zu waehlen.
     */
    private PanelAuswahl< SonderfertigkeitWert > panelSonderfertigkeitWert;

    /**
     * Die Schnittstelle die es erlaubt eine Sonderfertigkeit aus einer Liste
     * zu waehlen.
     */
    private PanelAuswahl< Sonderfertigkeit > panelSonderfertigkeit;

    /**
     * Die Benutzerschnittstelle mit der eine Auswahl fuer einen Talent-Bonus
     * getroffen werden kann. Bei einigen Talent-Boni kann der Benuter waehlen,
     * z.B. 'Schwerter oder Saebel +5'.
     */
    private PanelTalentBonus panelTalentBonus;

    /** Erlaubt es einen Vor-/Nachteil auszuwaehlen. */
    private PanelAuswahl< VorNachteilWert > panelVorNachteilWert;

    /** Ein Panel das es erlaubt eine Zweitsprache zu waehlen. */
    private PanelAuswahl< Sprache > panelZweitsprache;

    /**
     *
     * @param dasHauptfenster der Anwendung.
     */
    public Erschaffung( final Hauptpanel dasHauptpanel ) {

        hauptpanel = dasHauptpanel;

        panelErschaffung = hauptpanel.getPanelErschaffung();

        actionBeenden = new ActionBeenden();

        actionLeitzauber = new ActionLeitzauber();

        actionSonderfertigkeitBoniOffen =
            new ActionSonderfertigkeitBoniOffen();

        actionTalentBoni = new ActionTalentBoni();

        actionTalentBoniOffen = new ActionTalentBoniOffen();

        actionTalentBoniOffenProfession =
            new ActionTalentBoniOffenProfession();

        actionVerbilligteSonderfertigkeit =
            new ActionVerbilligteSonderfertigkeit();

        actionVorNachteilBoniOffen = new ActionVorNachteilBoniOffen();
    }

    /**
     * Setzt das {@link #panelTalentBonus} mit dessen Hilfe die Auswahl fuer
     * den uebergebenen Talent-Bonus getroffen wird und zeigt den Bonus im
     * Panel an.
     *
     * @param bonus fuer den der Benutzer als naechstes eine Auswahl treffen
     *      muss.
     */
    private final void anwendenTalentBonus( final Talentbonus bonus ) {

        switch( bonus.getTyp() ) {

        case FEST: {

            final PanelTalentBonusFest panel;
            panel = panelErschaffung.getPanelTalentBonusFest();

            panelTalentBonus = panel;

            panel.setBonus( manager.getHeld(), (TalentbonusFest) bonus );

            break;
        }
        case FREI: {

            final PanelTalentBonusFrei panel;
            panel = panelErschaffung.getPanelTalentBonusFrei();

            panelTalentBonus = panel;

            panel.setBonus( manager.getHeld(), (TalentbonusFrei) bonus );

            break;
        }
        default:
            throw new IllegalStateException( String.format(
                    "Die Methode Erschaffung#anwendenTalentBonus(TalentBonus)"+
                    " wurde mit einem TalentBonus aufgerufen der nicht vom " +
                    "Typ FEST oder FREI ist sondern vom Typ %1$s.",
                    bonus.getTyp() ) );
        }

    }

    /**
     * Der Teilschritt 1 bei der Erschaffung des Helden im Programm,
     * es werden Rasse, Kultur und Profession gewaehlt.
     */
    private final void schritt_1_Bausteine() {

        manager.getHeld().addObserver( actionTalentBoni );

        panelErschaffung.setWeiterAction( actionTalentBoni );

        final PanelRasKulProf panelRasKulProf =
            panelErschaffung.getPanelRasKulProf();

        panelRasKulProf.setHeld( manager.getHeld() );

        panelErschaffung.anzeigen( panelRasKulProf );

    }

    /**
     * Teilschritt 2.0 bei der Heldenerschaffung im Programm.
     * Es werden die Talent-Boni die der Held durch Rasse, Kulutur
     * und Profession erhaelt angewendet.
     */
    private final void schritt_2_0_TalentBoni() {

        panelErschaffung.getPanelRasKulProf().abmeldenVonHeld();

        // Muss hier ausgefuehrt werden, damit die KL eingestellt wird fuer
        // das Anzeigen des TaW der Zweit- und Muttersprache.
        manager.einstellenMindeststufe();

        manager.anwendenTalentBoniLeittalente();

        schritt_2_1_Zweitsprache();
    }

    /**
     * Teilschritt 2.1 bei der Heldenerschaffung im Programm,
     * wenn noetig kann eine Auswahl fuer die Zweitsprache des Helden
     * getroffen werden.
     */
    private final void schritt_2_1_Zweitsprache() {

        if ( null != panelZweitsprache ) {
            // Es wurde gerade ueber das panelZweitsprache die Zweitsprache
            // des Helden gewaehlt. Das gilt es jetzt auszuwerten.

            final Sprache zweitsprache;
            zweitsprache = panelZweitsprache.getGewaehltesElement();

            manager.setZweitsprache( zweitsprache );

            panelZweitsprache = null;
        }

        if ( manager.hatZweitsprachenAuswahl() ) {

            final List< Sprache > zweitsprachen = manager.getZweitsprachen();

            panelZweitsprache = new PanelAuswahl< Sprache >();

            panelZweitsprache.setElemente(
                    ZWEITSPRACHE_WAEHLEN,
                    zweitsprachen );

            panelErschaffung.anzeigen( panelZweitsprache );

            panelErschaffung.setWeiterAction( actionZweitsprache );

        } else {

            schritt_2_2_TalentBoniOffen();
        }
    }

    /**
     * Teilschritt 2.2 bei der Heldenerschaffung,
     * die offenen Talentboni die der Held durch Rasse und Kultur erhalten hat
     * werden abgearbeitet.
     */
    private final void schritt_2_2_TalentBoniOffen() {

        if ( null != panelTalentBonus ) {
                // Es wurde gerade ueber panelTalentBonus ein Talent-Bonus
            // verteilt. Die Auswahl gilt es jetzt auszuwerten.

            for ( final TalentWert wert : panelTalentBonus.getTalentBoni() ) {

                manager.einfuegenVorgegeben( wert );
            }

            panelTalentBonus = null;
        }

        if ( manager.hatTalentBonusOffen() ) {

            anwendenTalentBonus( manager.getTalentBonusOffen() );

            panelErschaffung.anzeigen( panelTalentBonus );

            panelErschaffung.setWeiterAction( actionTalentBoniOffen );

        } else {

            // Es ist kein offener Bonus aus Rasse oder Kultur mehr zu
            // bearbeiten. Auf zum naechsten Schritt.
            schritt_2_3_TalentBoniOffenProfession();

        }
    }

    /**
     * Teilschritt 2.3 der Heldenerschaffung, die offenen Talent-Boni die der
     * Held durch seine Profession erhalten hat sind abzuarbeiten.
     */
    private final void schritt_2_3_TalentBoniOffenProfession() {

        if ( null != panelTalentBonus ) {
            // Es wurde gerade ein Talent-Bonus ausgewaehlt. Die Asuwahl gilt
            // es nun auszuwerten.
            for ( final TalentWert wert : panelTalentBonus.getTalentBoni() ) {

                manager.einfuegenProfession( wert );

            }

            panelTalentBonus = null;
        }

        if ( manager.hatTalentBonusOffenProfession() ) {

            anwendenTalentBonus( manager.getTalentBonusOffenProfession() );

            panelErschaffung.anzeigen( panelTalentBonus );

            panelErschaffung.setWeiterAction( actionTalentBoniOffenProfession );

        } else if ( manager.hatTalentBonusEntdecker() ) {

            final TalentbonusFest bonus = manager.getTalentBonusEntdecker();

            final PanelTalentBonusFestEntdecker panel;
            panel = panelErschaffung.getPanelTalentBonusEntdecker();

            panel.setBonus( manager.getHeld(), bonus );

            panelTalentBonus = panel;

            panelErschaffung.anzeigen( panelTalentBonus );

            panelErschaffung.setWeiterAction( actionTalentBoniOffenProfession );

        } else {

            // Es ist kein offener Bonus aus der Profession mehr zu
            // bearbeiten. Auf zum naechsten Schritt.
            schritt_3_0_Sonderfertigkeiten();
        }
    }

    /**
     * Teilschritt 3.0 der Heldenerschaffung.
     * Hier erhaelt der Held die Sonderfertigkeiten.
     */
    private final void schritt_3_0_Sonderfertigkeiten() {

        manager.anwendenSonderfertigkeiten();

        schritt_3_1_SonderfertigkeitBoniOffen();
    }

    /**
     * Teilschritt 3.1 der Heldenerschaffung.
     * Der Benutzer kann nacheinander die Sonderfertigkeit-Boni abarbeiten fuer
     * die eine Auswahl getroffen werden muss.
     */
    private final void schritt_3_1_SonderfertigkeitBoniOffen() {

        if ( null != panelSonderfertigkeitWert ) {
            // Es wurde gerade eine Sonderfertigkeit gewaehlt.

            final SonderfertigkeitWert wert;
            wert = panelSonderfertigkeitWert.getGewaehltesElement();

            manager.einfuegenVorgegeben( wert );

            panelSonderfertigkeitWert = null;
        }

        if ( manager.hatSonderfertigkeitBonusOffen() ) {

            final SonderfertigkeitBonus bonus;
            bonus = manager.getSonderfertigkeitBonusOffen();

            final List< SonderfertigkeitWert > werte;
            werte = bonus.getWerte( manager.getHeld() );

            panelSonderfertigkeitWert = panelErschaffung.getPanelSonderfertigkeitWert();

            panelSonderfertigkeitWert.setElemente(
                    SONDERFERTIGKEIT_WAEHLEN,
                    werte );

            panelErschaffung.anzeigen( panelSonderfertigkeitWert );

            panelErschaffung.setWeiterAction( actionSonderfertigkeitBoniOffen );

        } else {

            // Alle Boni sind abgearbeitet. Es kann zum naechsten Schritt
            // ueber gegangen werden.
            schritt_3_2_VerbilligteSonderfertigkeitOffen();

        }
    }

    /**
     * Teilschritt 3.2 der Heldenerschaffung.
     * Hier werden die verbilligten Sonderfertigkeiten gewaehlt.
     */
    private final void schritt_3_2_VerbilligteSonderfertigkeitOffen() {

        if ( null != panelSonderfertigkeit ) {
            // Es wurde gerade eine Sonderfertigkeit gewaehlt.

            final Sonderfertigkeit sonderfertigkeit;
            sonderfertigkeit = panelSonderfertigkeit.getGewaehltesElement();

            manager.einfuegenVerbilligt( sonderfertigkeit );

            panelSonderfertigkeit = null;
        }

        if ( manager.hatVerbilligteSonderfertigkeitOffen() ) {

            final VerbilligteSonderfertigkeit verbilligung;
            verbilligung = manager.getVerbilligteSonderfertigkeitOffen();

            final Held held = manager.getHeld();

            final List< Sonderfertigkeit > sonderfertigkeiten;
            sonderfertigkeiten = verbilligung.getSonderfertigkeiten( held );

            panelSonderfertigkeit = panelErschaffung.getPanelSonderfertigkeit();

            panelSonderfertigkeit.setElemente(
                    VERBILLIGTE_SONDERFERTIGKEIT_WAEHLEN,
                    sonderfertigkeiten );

            panelErschaffung.anzeigen( panelSonderfertigkeit );

            panelErschaffung.setWeiterAction( actionVerbilligteSonderfertigkeit );

        } else {

            schritt_4_0_VorNachteile();
        }
    }

    /**
     * Teilschritt 4.0 der Erschaffung.
     * Der Held erhaelt die Vor-/Nachteile.
     */
    private final void schritt_4_0_VorNachteile() {

        manager.anwendenVorNachteile();

        schritt_4_1_VorNachteilBoniOffen();
    }

    /**
     * Teilschritt 4.1 der Heldenerschaffung.
     * Der Benutzer kann die Vor-/Nachteile waehlen.
     */
    private final void schritt_4_1_VorNachteilBoniOffen() {

        if ( null != panelVorNachteilWert ) {
            // Es wurde gerade ein Vor-/Nachteil gewaehlt.

            final VorNachteilWert wert;
            wert = panelVorNachteilWert.getGewaehltesElement();

            manager.einfuegenVorgegeben( wert );

            panelVorNachteilWert = null;
        }

        if ( manager.hatVorNachteilBonusOffen() ) {

            final VorNachteilBonus bonus;
            bonus = manager.getVorNachteilBonusOffen();

            final List< VorNachteilWert > werte;
            werte = bonus.getWerte();

            panelVorNachteilWert = panelErschaffung.getPanelVorNachteilWert();

            panelVorNachteilWert.setElemente(
                    VORNACHTEIL_WAEHLEN,
                    werte );

            panelErschaffung.anzeigen( panelVorNachteilWert );

            panelErschaffung.setWeiterAction( actionVorNachteilBoniOffen );

        } else {

            // Alle Boni sind abgearbeitet. Es kann zum naechsten Schritt
            // ueber gegangen werden.
            schritt_5_0_Zauber();

        }
    }

    /**
     * Teilschritt 5.0 der Erschaffung. Der Held erhaelt seine Zauber.
     */
    private final void schritt_5_0_Zauber() {

        manager.anwendenZauber();

        if ( manager.hatZusaetzlicheLeitzauber() ) {

            panelLeitzauber = panelErschaffung.getPanelLeitzauber();

            panelLeitzauber.setDaten(
                    manager.getHeld(),
                    manager.getAnzahlZusaetzlicheLeitzauber() );

            panelErschaffung.anzeigen( panelLeitzauber );

            panelErschaffung.setWeiterAction( actionLeitzauber );

        } else {

            schritt_6_Steigern();

        }
    }

    /**
     * Teilschritt 5.1 der Erschaffung, der Benutzer kann zusaetzliche
     * Leitzauber fuer den Helden waehlen.
     */
    private final void schritt_5_1_LeitzauberGewaehlt() {

        for( final Zauber z: panelLeitzauber.getGewaehlteZauber() ) {

            manager.einfuegenLeitzauber( z );
        }

        schritt_6_Steigern();
    }


    /** Geht zum letzten Schritt der Erschaffung weiter. */
    private final void schritt_6_Steigern() {

        panelErschaffung.setWeiterAction( actionBeenden );

        // muss hier zum 2ten mal ausgefuehrt werden, da sich die
        // Modifikationen auf die Eigenschaften durch die erhaltenen
        // Vor-/Nachteile geaendert haben koennen.
        manager.einstellenMindeststufe();

        final PanelSteigernErschaffung panelSteigern =
            panelErschaffung.getPanelSteigern();

        panelSteigern.setManager( manager );

        panelErschaffung.anzeigen( panelSteigern );

        hauptpanel.anpassenGroesse();
    }

    /** Startet eine neue Erschaffung. */
    public final void starten() {

        panelLeitzauber = null;
        panelSonderfertigkeit = null;
        panelSonderfertigkeitWert = null;
        panelTalentBonus = null;
        panelVorNachteilWert = null;
        panelZweitsprache = null;

        manager = new ErschaffungManager();

        final PanelGp panelGp = panelErschaffung.getPanelGp();

        panelGp.setErschaffungManager( manager );

        hauptpanel.anzeigen( panelErschaffung, Warnen.WARNEN );

        schritt_1_Bausteine();
    }

    /**
     * Eine Hilfsfunktion fuer die Entwicklung und Debugging. Erschafft einen
     * Helden mit den uebergebenen Bausteinen. Die Boni bei denen eine Auswahl
     * getroffen werden muss, werden ignoriert.
     */
    public final ErschaffungManager schnellerschaffung(
            final Rasse rasse,
            final Kultur kultur,
            final Profession profession ) {

        manager = new ErschaffungManager();

        final Held held = manager.getHeld();

        held.setRasse( rasse );
        held.setKultur( kultur );
        held.setProfession( profession );

        manager.anwendenSonderfertigkeiten();
        manager.anwendenTalentBoniLeittalente();
        manager.anwendenVorNachteile();
        manager.anwendenZauber();
        manager.einstellenMindeststufe();

        held.setWirdErschaffen( false );

        return manager;
    }

    /**
     * Beendet die Erschaffung nach dem Steigern.
     *
     * @author Bernhard Jung
     *
     */
    private class ActionBeenden extends AbstractAction {

        ActionBeenden() {
            super( BEENDEN_NAME );

            putValue( MNEMONIC_KEY, Util.getKeyCode( BEENDEN_MNEMONIC ) );
        }

        public void actionPerformed( final ActionEvent e ) {

            final Held held = manager.getFertigenHeld();

            final PanelSteigern panelSteigern = hauptpanel.getPanelSteigern();

            panelSteigern.setHeld( held );

            hauptpanel.anzeigen( panelSteigern, Warnen.NICHT_WARNEN );
        }
    }

    /**
     * Die Action die verwendet wird wenn die zusaetzlichen Leitzauber gewaehlt
     * werden.
     */
    private class ActionLeitzauber extends AbstractAction {

        public ActionLeitzauber() {
            super( WEITER );

            putValue( MNEMONIC_KEY, Util.getKeyCode( WEITER_MNEMONIC ) );
        }

        public void actionPerformed( final ActionEvent e ) {
            schritt_5_1_LeitzauberGewaehlt();
        }

    }

    /**
     * Die Action die verwendet wird wenn nacheinander die offenen Auswahlen
     * fuer die Sonderfertigkeiten abgearbeitet werden.
     */
    private class ActionSonderfertigkeitBoniOffen extends AbstractAction {

        public ActionSonderfertigkeitBoniOffen() {
            super( WEITER );

            putValue( MNEMONIC_KEY, Util.getKeyCode( WEITER_MNEMONIC ) );
        }

        public void actionPerformed( final ActionEvent e ) {
            schritt_3_1_SonderfertigkeitBoniOffen();
        }

    }

    /**
     * Die Action die verwendet wird wenn nacheinander die offenen Talent-Boni
     * abgearbeitet werden, die der Held durch Rasse und Kultur erhalten hat.
     */
    private class ActionTalentBoniOffen extends AbstractAction {

        public ActionTalentBoniOffen() {
            super( WEITER );

            putValue( MNEMONIC_KEY, Util.getKeyCode( WEITER_MNEMONIC ) );
        }

        public void actionPerformed( final ActionEvent e ) {
            schritt_2_2_TalentBoniOffen();
        }

    }

    /**
     * Die Action die verwendet wird wenn nacheinander die offenen Talent-Boni,
     * die der Held durch seine Profession erhalten hat, abgearbeitet werden.
     */
    private class ActionTalentBoniOffenProfession extends AbstractAction {

        public ActionTalentBoniOffenProfession() {
            super( WEITER );

            putValue( MNEMONIC_KEY, Util.getKeyCode( WEITER_MNEMONIC ) );
        }

        public void actionPerformed( final ActionEvent e ) {
            schritt_2_3_TalentBoniOffenProfession();
        }

    }

    /**
     * Die Action die zum 2ten Schritt der Erschaffung, der Auswahl der
     * Talent-Boni, fuehrt.
     */
    private class ActionTalentBoni extends AbstractAction implements Observer {

        private ActionTalentBoni() {
            super( WEITER );

            putValue( MNEMONIC_KEY, Util.getKeyCode( WEITER_MNEMONIC ) );

            setEnabled( false );
        }

        public final void actionPerformed( final ActionEvent e ) {
            schritt_2_0_TalentBoni();
        }

        /**
         * @param o der Held der ueberwacht wird.
         * @param arg ungenutzt.
         */
        public final void update( final Observable o, final Object arg ) {

            final Held held = (Held) o;

            setEnabled( ( held.getRasse()   != Rasse.LEERE_RASSE )
                  && ( held.getKultur()     != Kultur.LEERE_KULTUR )
                  && ( held.getProfession() != Profession.LEERE_PROFESSION ) );
        }
    }

    /**
     * Die Action die verwendet wird wenn die offenen Auswahlen
     * fuer die verbilligte Sonderfertigkeit abgearbeitet werden.
     */
    private class ActionVerbilligteSonderfertigkeit extends AbstractAction {

        public ActionVerbilligteSonderfertigkeit() {
            super( WEITER );

            putValue( MNEMONIC_KEY, Util.getKeyCode( WEITER_MNEMONIC ) );
        }

        public void actionPerformed( final ActionEvent e ) {
            schritt_3_1_SonderfertigkeitBoniOffen();
        }

    }

    /**
     * Die Action die verwendet wird um die offenen Auswahlen fuer die
     * Vor-/Nachteile abzuarbeiten.
     */
    private class ActionVorNachteilBoniOffen extends AbstractAction {

        public ActionVorNachteilBoniOffen() {
            super( WEITER );

            putValue( MNEMONIC_KEY, Util.getKeyCode( WEITER_MNEMONIC ) );
        }

        public void actionPerformed( final ActionEvent e ) {
            schritt_4_1_VorNachteilBoniOffen();
        }

    }

    /**
     * Die Action die verwendet wird um die Zweitsprache des Helden zu waehlen.
     */
    private class ActionZweitsprache extends AbstractAction {

        public ActionZweitsprache() {
            super( WEITER );

            putValue( MNEMONIC_KEY, Util.getKeyCode( WEITER_MNEMONIC ) );
        }

        public void actionPerformed( final ActionEvent e ) {
            schritt_2_1_Zweitsprache();
        }

    }

}
