/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Locale;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;

/**
 * Der Dialog um die Einstellungen fuer das Programm zu waehlen. Das sind alle
 * Einstellungen die nichts mit den Regeln zu tun haben.
 *
 */
public class DialogEinstellungen extends JDialog {
    
    /** Der Text fuer den Button mit dem der Dialog geschlossen wird. */
    private static final String SCHLIESSEN =
            I18n.i18n( "DialogEinstellungen.Schliessen" );
    /**
     * Der Hinweis an den Benutzer, dass eine Aenderung der Spracheinstellunge
     * einen Neustart benotigt.
     */
    private static final String SPRACHAENDERUNG_BENOETIGT_NEUSTART =
        I18n.i18n( "DialogEinstellungen.SprachaenderungBenoetigtNeustart" );
    
    /**
     * Der Text vor der Combobox mit der die Sprache fuer das Programm gewaehlt
     * wird.
     */
    private static final String SPRACHE =
        I18n.i18n( "DialogEinstellungen.Sprache" );

    /**
     * Der Titel fuer diesen Dialog. 
     */
    private static final String TITEL = 
        I18n.i18n( "DialogEinstellungen.Titel" ); 
    
    /**
     * Hier muessen die Sprachen eingetragen werden, fuer die es eine 
     * Uebersetzung des Programms gibt.
     */
    private static final Locale[] LOCALES = {
        new Locale( "de" ),
        new Locale( "en" )
    };
    
    public DialogEinstellungen( final JFrame hauptfenster ) {
        
        super( hauptfenster, TITEL );
        
        final Einstellungen einstellungen = Einstellungen.getInstanz();
        
        final JLabel sprache = new JLabel( SPRACHE );
        
        final JComboBox spracheComboBox = erzeugeSpracheComboBox();
        
        final JCheckBox spracheCheckBox = new JCheckBox();
        spracheCheckBox.setSelected( einstellungen.isSpracheGesetzt() );
        spracheCheckBox.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                einstellungen.setSpracheGesetzt( spracheCheckBox.isSelected() );
                spracheComboBox.setEnabled( spracheCheckBox.isSelected() );
            }
        });
        
        final JLabel sprachaenderungBenoetigtNeustart =  
            new JLabel( SPRACHAENDERUNG_BENOETIGT_NEUSTART ); 
        
        final JButton schliessen = new JButton( SCHLIESSEN );
        schliessen.addActionListener( new ActionListener() {
           public void actionPerformed(ActionEvent e) {
               setVisible( false );
           } 
        });
        final JPanel buttonBar = ButtonBarFactory.buildCloseBar( schliessen );
        
        final FormLayout layout = new FormLayout( 
                "7dlu, P, 4dlu, P, 2dlu, P, 2dlu, P:G, 7dlu",
                "7dlu, P, 7dlu, P, 7dlu" );
        
        setLayout( layout );
        
        final CellConstraints cc = new CellConstraints();
        
        add( sprache, cc.xy( 2, 2 ) );
        
        add( spracheCheckBox, cc.xy( 4, 2 ) );
        
        add( spracheComboBox, cc.xy( 6, 2 ) );
        
        add( sprachaenderungBenoetigtNeustart, cc.xy( 8, 2 ) );
        
        add( buttonBar, cc.xyw( 2, 4, 7 ) );
        
        pack();
    }
    
    /**
     * Erzeugt die ComboBox mit den Sprachen fuer die das Programm lokalisiert
     * ist.
     * 
     * @return Die fertige ComboBox. Die aktuelle Sprache ist schon eingestellt. 
     */
    private JComboBox erzeugeSpracheComboBox() {
        
        final Einstellungen einstellungen = Einstellungen.getInstanz();
        
        final Vector< String > sprachen = new Vector< String >( LOCALES.length );
        
        for ( Locale locale : LOCALES ) {
            
            sprachen.add( locale.getDisplayLanguage() );
        }
        
        final JComboBox comboBox = new JComboBox( sprachen );
        
        comboBox.setEnabled( einstellungen.isSpracheGesetzt() );
        
        comboBox.setSelectedItem( 
                einstellungen.getSprache().getDisplayLanguage() );
        
        comboBox.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int index = comboBox.getSelectedIndex();
                
                einstellungen.setSprache( LOCALES[ index ] );
            }
        });
        
        return comboBox;
    }
}
