/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.gui.bonus.PanelAuswahl;
import de.bernhardjung.dsaprogramm.gui.bonus.PanelLeitzauber;
import de.bernhardjung.dsaprogramm.gui.bonus.PanelTalentBonusFest;
import de.bernhardjung.dsaprogramm.gui.bonus.PanelTalentBonusFestEntdecker;
import de.bernhardjung.dsaprogramm.gui.bonus.PanelTalentBonusFrei;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

public class PanelErschaffung extends JPanel {

    /**
     * Erklaert dem Benutzer, dass er zu einem Wert in rot, über einen Tooltip,
     * nähere Informationen erhalten kann.
     */
    private static final String HINWEIS =
        I18n.i18n( "PanelErschaffung.Hinweis" );

    /**
     * Der Knopf mit dem man zum naechsten Schritt der Erschaffung gelangen
     * kann.
     */
    private final JButton knopfWeiter;

    /** Die Benutzerschnittstelle, die aktuell im Panel angezeigt wird. */
    private JComponent hauptcomponent;

    /** Das Panel auf dem die verbliebenen AP und GP angezeigt werden. */
    private PanelGp panelGp;

    /**
     * Das Panel mit dem Knopf, mit dem man bei der Erschaffung zum naechsten
     * Schritt ueber geht.
     */
    private JPanel panelKnopfWeiter;

    /**
     * Das Panel mit dem die zusaetzlichen Leitzauber fuer einen elfischen
     * Helden gewaehlt werden koennen.
     */
    private PanelLeitzauber panelLeitzauber;

    /** Das Panel um die Rasse, Kultur und Profession zu waehlen. */
    private PanelRasKulProf panelRasKulProf;

    /**
     * Ein Panel mit dem man eine Sonderfertigkeit aus einer Liste waehlen
     * kann.
     */
    private
    PanelAuswahl< SonderfertigkeitWert > panelSonderfertigkeitWert;

    /**
     * Ein Panel mit dem man eine Sonderfertigkeit aus einer Liste waehlen
     * kann.
     */
    private PanelAuswahl< Sonderfertigkeit > panelSonderfertigkeit;

    /**
     * Das Panel zum steigern des Helden nachdem Rasse, Kultur und Profession
     * gewaehlt worden sind.
     */
    private PanelSteigernErschaffung panelSteigern;

    /**
     * Das Panel mit dem eine Auswahl fuer einen Bonus wie 'Singen, Tanzen
     * oder Zechen +5' getroffen werden kann.
     */
    private PanelTalentBonusFest panelTalentBonusFest;

    /**
     * Das Panel mit dem eine Auswahl fuer den speziellen Bonus, den die
     * Profession Entdecker vergibt, getroffen werden kann.
     */
    private PanelTalentBonusFestEntdecker panelTalentBonusEntdecker;

    /**
     * Das Panel mit dem eine Auswahl fuer einen Bonus wie '4 Punkte auf
     * Oloarkh, Ologhaijan und/oder Thorwalsch verteilen' getroffen werden
     * kann.
     */
    private PanelTalentBonusFrei panelTalentBonusFrei;

    /**
     * Das Panel mit dem man einen Vor-/Nachteil fuer den Helden waehlen kann.
     */
    private PanelAuswahl< VorNachteilWert > panelVorNachteilWert;

    public PanelErschaffung() {

        super( new BorderLayout() );

        knopfWeiter = new JButton();

        add( getPanelGp(), BorderLayout.PAGE_START );

        add( getPanelKnopfWeiter(), BorderLayout.PAGE_END );
    }

    /**
     * Bestimmt was im Zentrum des Hauptfensters dargestellt wird.
     *
     * @param component die dargestellt wird.
     */
    public final void anzeigen( final JComponent component ) {

        if ( null != component
         && component != hauptcomponent ) {

            if ( null != hauptcomponent ) {

                remove( hauptcomponent );

            }

            hauptcomponent = component;

            add( hauptcomponent, BorderLayout.CENTER );

            validate();
            repaint();
        }
    }

    /**
     * @return das Panel auf dem die noch zu verteilenden AP und GP dargestellt
     *      werden.
     */
    public final PanelGp getPanelGp() {
        if ( null == panelGp ) {
            panelGp = new PanelGp();
        }
        return panelGp;
    }

    /**
     * @return das Panel mit dem die zusaetzlichen Leitzauber fuer einen
     *         elfischen Helden gewaehlt werden koennen.
     */
    public final PanelLeitzauber getPanelLeitzauber() {
        if ( null == panelLeitzauber ) {
            panelLeitzauber = new PanelLeitzauber();
        }
        return panelLeitzauber;
    }

    /**
     * @return das Panel mit dem Rasse, Kultur und Profession gewaehlt werden.
     */
    public final PanelRasKulProf getPanelRasKulProf() {
        if ( null == panelRasKulProf ) {
            panelRasKulProf = new PanelRasKulProf();
        }
        return panelRasKulProf;
    }

    /**
     * @return das Panel mit dem man eine Sonderfertigkeit aus einer Liste
     *      waehlen kann.
     */
    public final PanelAuswahl< Sonderfertigkeit > getPanelSonderfertigkeit() {
        if ( null == panelSonderfertigkeit ) {
            panelSonderfertigkeit = new PanelAuswahl< Sonderfertigkeit >();
        }
        return panelSonderfertigkeit;
    }

    /**
     * @return das Panel mit dem man eine Sonderfertigkeit aus einer Liste
     *      waehlen kann.
     */
    public final
    PanelAuswahl< SonderfertigkeitWert > getPanelSonderfertigkeitWert() {
        if ( null == panelSonderfertigkeitWert ) {
            panelSonderfertigkeitWert =
                new PanelAuswahl< SonderfertigkeitWert >();
        }
        return panelSonderfertigkeitWert;
    }

    /**
     * @return das Panel mit dem im letzten Schritt der Erschaffung die Werte
     *      des Helden gesteigert werden.
     */
    public final PanelSteigernErschaffung getPanelSteigern() {
        if ( null == panelSteigern ) {
            panelSteigern = new PanelSteigernErschaffung();
        }
        return panelSteigern;
    }

    /**
     * @return das Panel mit dem eine Auswahl fuer den speziellen Bonus
     *      getroffen werden kann, welchen die Profession Entdecker vergibt.
     */
    public final PanelTalentBonusFestEntdecker getPanelTalentBonusEntdecker() {
        if ( null == panelTalentBonusEntdecker ) {
            panelTalentBonusEntdecker = new PanelTalentBonusFestEntdecker();
        }
        return panelTalentBonusEntdecker;
    }

    /**
     * @return das Panel mit dem einen Auswahl fuer einen Bonus wie '4 Punkte
     *      auf Oloarkh, Ologhaijan oder Thorwalsch verteilen' getroffen werden
     *      kann.
     */
    public final PanelTalentBonusFest getPanelTalentBonusFest() {
        if ( null == panelTalentBonusFest ) {
            panelTalentBonusFest = new PanelTalentBonusFest();
        }
        return panelTalentBonusFest;
    }

    /**
     * @return das Panel mit dem eine Auswahl fuer einen Bonus wie
     *      'Singen, Tanzen oder Zechen +5' getroffen werden kann.
     */
    public final PanelTalentBonusFrei getPanelTalentBonusFrei() {
        if ( null == panelTalentBonusFrei ) {
            panelTalentBonusFrei = new PanelTalentBonusFrei();
        }
        return panelTalentBonusFrei;
    }

    /**
     * @return das Panel mit dem man einen Vor-/Nachteil auswaehlen kann.
     */
    public final PanelAuswahl< VorNachteilWert > getPanelVorNachteilWert() {
        if ( null == panelVorNachteilWert ) {
            panelVorNachteilWert = new PanelAuswahl< VorNachteilWert >();
        }
        return panelVorNachteilWert;
    }

    public final JPanel getPanelKnopfWeiter() {
        if ( null == panelKnopfWeiter ) {
            panelKnopfWeiter = new JPanel(
                    new FormLayout( "7dlu, F:P:G, R:P:G, 7dlu", "7dlu, P, 7dlu" ) );
            final CellConstraints cc = new CellConstraints();
            panelKnopfWeiter.add( new JLabel( HINWEIS ), cc.xy( 2, 2 ) );
            panelKnopfWeiter.add( knopfWeiter, cc.xy( 3, 2 ) );
        }
        return panelKnopfWeiter;
    }

    /**
     * @param action mit welcher der Knopf versehen wird der rechts unten
     *      im Fenster zu sehen ist.
     */
    public final void setWeiterAction( final Action action ) {

        knopfWeiter.setAction( action );
    }
}
