/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import static de.bernhardjung.dsaprogramm.i18n.I18n.i18n;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Augenfarbe;
import de.bernhardjung.dsaprogramm.basis.Haarfarbe;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.bausteine.util.Geschlecht;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Erlaubt es die allgemeinen Informationen ueber den Helden
 * anzuzeigen und zu editieren. Es geht um solche Dinge wie Name, Geschlecht,
 * Koerpergroesse usw.
 *
 * @author Bernhard Jung
 */
public class PanelHeldendaten extends JPanel implements Observer {

    /**
     * Ein Spinner mit dem man das Alter des Helden in Jahren einstellen kann.
     */
    private final JSpinner alter;

    /**
     * Ein Knopf mit dem man das Alter des Helden auswuerfeln lassen kann.
     */
    private final JButton alterAuswuerfeln;

    /** Eine ComboBox in der man die Augenfarbe waehlen kann. */
    private final JComboBox augenfarbe;

    /** Ein Knopf um die Augenfarbe auswuerfeln zu lassen */
    private final JButton augenfarbeAuswuerfeln;

    /**
     * Ein Textfeld in das eine kurze Beschreibung des Aussehens des Helden
     * eingetragen werden kann.
     */
    private final JTextField aussehn;

    /**
     * Ein Textfeld in das der Geburtstag des Helden eingetragen werden kann.
     */
    private final JTextField geburtstag;

    /**
     * Eine Combobox mit der das Geschlecht des Helden/der Heldin gewaehlt
     * werden kann.
     */
    private final JComboBox geschlecht;

    /**
     * Ein Spinner mit dem das Gewicht des Helden gewaehlt werden kann.
     */
    private final JSpinner gewicht;

    /**
     * Ein Knopf um das Gewicht des Helden auszwuerfeln.
     * Die Option ist nötig für den Fall, dass der Held fettleibig ist.
     */
    private final JButton gewichtAuswuerfeln;

    /** Ein Spinner um die Groesse des Helden zu waehlen. */
    private final JSpinner groesse;

    /** Ein Knopf um die groesse des Helden auswuerfen zu lassen */
    private final JButton groesseAuswuerfeln;

    /** Eine Combobox mit der die Haarfarbe des Helden gewaehlt werden kann. */
    private final JComboBox haarfarbe;

    /** Ein Knopf um die Haarfarbe des Helden auswuerfeln zu lassen. */
    private final JButton haarfarbeAuswuerfeln;

    /** Der Held der gerade erschaffen wird. */
    private Held held;

    /**
     * Ein Textfeld in das Informationen zu Familie, Herkunft und Hintergrund
     * des Helden eingegeben werden koennen.
     */
    private final JTextField hintergrund;

    /** Das Label das den Namen der Kultur des Helden anzeigt. */
    private final JLabel kultur;

    /** Ein Textfeld in das der Name des Helden eingegeben werden kann. */
    private final JTextField name;

    /** Ein Label das den Namen der Profession des Helden anzeigt. */
    private final JLabel profession;

    /** Ein Label das den Namen der Rasse des Helden anzeigt. */
    private final JLabel rasse;

    /** Ein Texfeld in das der Stand des Helden eingetragen werden kann. */
    private final JTextField stand;

    /** Ein Texfeld in das der Titel des Helden eingetragen werden kann. */
    private final JTextField titel;

    public PanelHeldendaten() {

        alter = Util.erzeugeSpinner( 1, 1, 999, 1 );
        alter.addChangeListener( new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                held.setAlter( (Integer) alter.getValue() );
            }
        } );

        alterAuswuerfeln = new JButton(
                I18n.i18n( "PanelHeldendaten.Auswuerfeln" ) );
        alterAuswuerfeln.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {
                held.setAlter( held.getRasse().getAlter().auswuerfeln( held ) );
            }
        });

        augenfarbe = new JComboBox( Augenfarbe.getAugenfarben() );
        augenfarbe.setEditable( true );
        augenfarbe.addKeyListener( new KeyAdapter() {
            @Override
            public void keyTyped( final KeyEvent e ) {
                held.setAugenfarbe( (String) augenfarbe.getSelectedItem() );
            }
        } );
        augenfarbe.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {
                held.setAugenfarbe( (String) augenfarbe.getSelectedItem() );
            }
        } );

        augenfarbeAuswuerfeln =
            new JButton( i18n( "PanelHeldendaten.Auswuerfeln" ) );
        augenfarbeAuswuerfeln.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                final String augenfarbe;
                augenfarbe = held.getRasse().getAugenfarben().wuerfeln();
                held.setAugenfarbe( augenfarbe );
            }
        } );

        aussehn = new JTextField();
        aussehn.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                held.setAussehen( aussehn.getText() );
            }
        } );

        geburtstag = new JTextField();
        geburtstag.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                held.setGeburtstag( geburtstag.getText() );
            }
        } );

        geschlecht = new JComboBox( Geschlecht.values() );
        geschlecht.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                held.setGeschlecht(
                        (Geschlecht) geschlecht.getSelectedItem() );
            }
        } );

        gewicht = Util.erzeugeSpinner( 1, 1, Integer.MAX_VALUE, 1 );
        gewicht.addChangeListener( new ChangeListener() {
            public void stateChanged( final ChangeEvent e ) {
                held.setGewicht( (Integer) gewicht.getValue() );
            }
        } );

        gewichtAuswuerfeln =
            new JButton( i18n( "PanelHeldendaten.Auswuerfeln" ) );
        gewichtAuswuerfeln.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                held.setGewicht( held.getRasse().getGewicht().berechnen( held));
            }
        } );

        groesse = Util.erzeugeSpinner( 1, 1, Integer.MAX_VALUE, 1 );
        groesse.addChangeListener( new ChangeListener() {
            public void stateChanged( final ChangeEvent e ) {
                held.setGroesse( (Integer) groesse.getValue() );
                final int dasGewicht;
                dasGewicht = held.getRasse().getGewicht().berechnen( held );
                held.setGewicht( dasGewicht );
            }
        } );

        groesseAuswuerfeln =
            new JButton( i18n( "PanelHeldendaten.Auswuerfeln" ) );
        groesseAuswuerfeln.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                held.setGroesse( held.getRasse().getGroesse().wuerfeln() );
            }
        } );

        haarfarbe = new JComboBox( Haarfarbe.getHaarfarben() );
        haarfarbe.setEditable( true );
        haarfarbe.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                held.setHaarfarbe( (String) haarfarbe.getSelectedItem() );
            }
        } );
        haarfarbe.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                held.setHaarfarbe( (String) haarfarbe.getSelectedItem() );
            }
        } );

        haarfarbeAuswuerfeln =
            new JButton( i18n( "PanelHeldendaten.Auswuerfeln" ) );
        haarfarbeAuswuerfeln.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                final String farbe;
                farbe = held.getRasse().getHaarfarben().wuerfeln();
                held.setHaarfarbe( farbe );
            }
        } );

        held = new Held();

        hintergrund = new JTextField();
        hintergrund.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                held.setHintergrund( hintergrund.getText() );
            }
        } );

        kultur = new JLabel();

        name = new JTextField();
        name.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                held.setName( name.getText() );
            }
        } );

        profession = new JLabel();

        rasse = new JLabel();

        stand = new JTextField();
        stand.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                held.setStand( stand.getText() );
            }
        } );

        titel = new JTextField();
        titel.addFocusListener( new FocusAdapter() {
            @Override
            public void focusLost( final FocusEvent e ) {
                held.setTitel( titel.getText() );
            }
        } );

        aufbauenBenutzeroberflaeche();
    }

    /** Baut die Benutzeroberlfaeche auf. Ordnet alle Komponenten an. */
    private final void aufbauenBenutzeroberflaeche() {

        final FormLayout layout = new FormLayout(
              // 1    2        3          4   5    6   7
                "M, 4dlu, 30dlu:G(0.5), 2dlu, M, 4dlu, M, " + // 1. Hauptspalte
                "8dlu, " +
                "M, 4dlu, 30dlu:G(0.5), 2dlu, M, 4dlu, M"    // 2. Hauptspalte
              // 9   10       11         12  13   14  15
                ); // Zeilen werden vom Builder erzeugt.

        // Die 1. Spalte in der linken Hauptspalte soll die gleiche Breite
        // haben wie die 1. in der rechten Hauptspalte.
        // Gleiches gilt fuer alle anderen Spalten.
        layout.setColumnGroups( new int[][]{ { 1,  9 },
                                             { 3, 11 },
                                             { 5, 13 },
                                             { 7, 15 } } );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout, this );
        // ein schoener breiter Rahmen um alles
        builder.setDefaultDialogBorder();
        builder.setRowGroupingEnabled( true ); // alle Zeilen gleich hoch.

        // 1. Zeile
        builder.appendTitle( i18n( "PanelHeldendaten.Name" ) );
        builder.append( name, 13 );

        // naechste Zeile
        builder.appendTitle( i18n( "PanelHeldendaten.Rasse" ) );
        builder.append( rasse, 13 );

        // naechste Zeile
        builder.appendTitle( i18n( "PanelHeldendaten.Kultur" ) );
        builder.append( kultur, 13 );

        // naechste Zeile
        builder.appendTitle( i18n( "PanelHeldendaten.Profession" ) );
        builder.append( profession, 13 );

        // naechste Zeile
        // linke Haelfte
        builder.appendTitle( i18n( "PanelHeldendaten.Geschlecht" ) );
        builder.append( geschlecht, 5 );
        // rechte Haelfte
        builder.appendTitle( i18n( "PanelHeldendaten.Geburtstag" ) );
        builder.append( geburtstag, 5 );

        // naechste Zeile
        // linke Haelfte
        builder.append( new JLabel(), 7 );
        // rechte Haelfte
        builder.appendTitle( i18n( "PanelHeldendaten.Alter" ) );
        builder.append( alter );
        builder.append( new JLabel( i18n( "PanelHeldendaten.Jahre" ) ) );
        builder.append( alterAuswuerfeln );
        builder.nextLine();

        // naechste Zeile
        // linke Haelfte
        builder.appendTitle( i18n( "PanelHeldendaten.Haarfarbe" ) );
        builder.append( haarfarbe, 3 );
        builder.append( haarfarbeAuswuerfeln );
        // rechte Haelfte
        builder.appendTitle( i18n( "PanelHeldendaten.Groesse" ) );
        builder.append( groesse );
        builder.append( i18n( "PanelHeldendaten.Halbfinger" ) );
        builder.append( groesseAuswuerfeln );

        // naechste Zeile
        // linke Haelfte
        builder.appendTitle( i18n( "PanelHeldendaten.Augenfarbe" ) );
        builder.append( augenfarbe, 3 );
        builder.append( augenfarbeAuswuerfeln );
        // rechte Haelfte
        builder.appendTitle( i18n( "PanelHeldendaten.Gewicht" ) );
        builder.append( gewicht );
        builder.append( new JLabel( i18n( "PanelHeldendaten.Stein" ) ) );
        builder.append( gewichtAuswuerfeln );

        // naechste Zeile
        builder.appendTitle( i18n( "PanelHeldendaten.Aussehen" ) );
        builder.append( aussehn, 13 );

        // naechste Zeile
        builder.appendTitle( i18n( "PanelHeldendaten.Stand" ) );
        builder.append( stand, 13 );

        // naechste Zeile
        builder.appendTitle( i18n( "PanelHeldendaten.Titel" ) );
        builder.append( titel, 13 );

        // naechste Zeile
        builder.appendTitle( i18n( "PanelHeldendaten.Hintergrund" ) );
        builder.append( hintergrund, 13 );

    }

    /**
     * Bestimmt den Held der gerade erschaffen wird.
     * Die Werte des Helden werden angezeigt und koennen in Folge ueber das
     * Panel editiert werden.
     *
     * @param neuerHeld der gerade erschaffen wird.
     */
    public final void setHeld( final Held neuerHeld ) {

        held = neuerHeld;

        held.addObserver( this );

        final Rasse rasse = held.getRasse();

        alterAuswuerfeln.setToolTipText( rasse.getAlter().toString() );

        augenfarbeAuswuerfeln.setToolTipText( "<html>" +
                rasse.getAugenfarben().toHtmlTabelleVertikal() );

        gewichtAuswuerfeln.setToolTipText( rasse.getGewicht().toString() );

        groesseAuswuerfeln.setToolTipText( rasse.getGroesse().toString() );

        haarfarbeAuswuerfeln.setToolTipText( "<html>" +
                rasse.getHaarfarben().toHtmlTabelleVertikal() );

        kultur.setText( held.getKultur().getName() );

        profession.setText( held.getProfession().getName( held ) );

        this.rasse.setText( rasse.getName() );

        update( null, null );
    }

    /**
     * Aktualisiert die Anzeige wenn Aenderungen am Helden stattgefunden haben.
     * @param o ungenutzt
     * @param arg ungenutzt
     */
    public void update( final Observable o, final Object arg ) {

        KonfliktBeschreibung beschreibung = held.getRasse().pruefen( held );

        if ( beschreibung.enthaeltKonflikt() ) {

            rasse.setForeground( Util.FEHLERFARBE );

            rasse.setToolTipText( beschreibung.toHtml() );

        } else {

            rasse.setForeground( getForeground() );

            rasse.setToolTipText( null );
        }

        beschreibung = held.getKultur().pruefen( held );

        if ( beschreibung.enthaeltKonflikt() ) {

            kultur.setForeground( Util.FEHLERFARBE );

            kultur.setToolTipText( beschreibung.toHtml() );

        } else {

            kultur.setForeground( getForeground() );

            kultur.setToolTipText( null );

        }

        beschreibung = held.getProfession().pruefen( held );

        if ( beschreibung.enthaeltKonflikt() ) {

            profession.setForeground( Util.FEHLERFARBE );

            profession.setToolTipText( beschreibung.toHtml() );

        } else {

            profession.setForeground( getForeground() );

            profession.setToolTipText( null );
        }

        profession.setText( held.getProfession().getName( held ) );

        alter.setValue( held.getAlter() );

        augenfarbe.setSelectedItem( held.getAugenfarbe() );

        geburtstag.setText( held.getGeburtstag() );

        geschlecht.setSelectedItem( held.getGeschlecht() );

        gewicht.setValue( held.getGewicht() );

        gewichtAuswuerfeln.setEnabled( held.hat( VorNachteil.FETTLEIBIG ) );

        groesse.setValue( held.getGroesse() );

        haarfarbe.setSelectedItem( held.getHaarfarbe() );

        name.setText( held.getName() );
    }

}
