/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Fasst 3 Panel zusammen um Rasse, Kultur und Profession waehlen zu koennen.
 */
public class PanelRasKulProf extends JPanel {

    /** Das Panel mit dem die Kultur gewaehlt wird. */
    private final PanelKultur panelKultur;

    /** Das Panel mit dem die Profession gewaehlt wird. */
    private final PanelProfession panelProfession;

    /** Das Panel mit dem die Rasse gewaehlt wird. */
    private final PanelRasse panelRasse;

    PanelRasKulProf() {

        super( new BorderLayout() );

        panelProfession = new PanelProfession();

        panelKultur = new PanelKultur( panelProfession );

        panelRasse = new PanelRasse( panelKultur );

        final JTabbedPane tabbedPane = new JTabbedPane();

        tabbedPane.addTab( I18n.i18n( "PanelRasKulProf.Rasse" ), panelRasse );
        tabbedPane.addTab(
                I18n.i18n( "PanelRasKulProf.Kultur" ), panelKultur );
        tabbedPane.addTab(
                I18n.i18n( "PanelRasKulProf.Profession" ), panelProfession );

        add( tabbedPane );
    }

    /**
     * Meldet das Panel als Observer des Helden ab.
     */
    public final void abmeldenVonHeld() {

        panelRasse.abmeldenVonHeld();

        panelKultur.abmeldenVonHeld();

        panelProfession.abmeldenVonHeld();
    }

    /**
     * Aendert den {@link Held} fuer den Rasse, Kultur und Profession gewaehlt
     * werden. Meldet das Panel als Observer beim Helden an.
     *
     * @see #abmeldenVonHeld()
     * @param held
     *            der gerade erschaffen wird.
     */
    public final void setHeld( final Held held ) {

        panelRasse.setHeld( held );

        panelKultur.setHeld( held );

        panelProfession.setHeld( held );
    }
}
