/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.erschaffung.ErschaffungManager;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeitgruppe;

/**
 * Eine Benutzerschnittstelle die alle Sonderfertigkeiten anzeigt, die die der
 * Held schon besitzt und die die er noch waehlen kann, und es erlaubt die
 * Sonderfertigkeiten eines Helden zu veraendern.
 * <p>
 * Die Schnittstelle besteht aus einem Register das unterschiedliche Listen mit
 * Sonderfertigkeiten anzeigt. Wird das Panel beim Steigern eines Helden
 * verwendet, so wird ueber dem Register eine Combobox angezeigt mit der man
 * eine Lernmethode waehlen kann.
 *
 * @author Bernhard Jung
 */
public class PanelSonderfertigkeiten extends JPanel {

    /** Der Text neben der Combobox mit der man die Lernmethode waehlt. */
    private static final String LERNMETHODE =
        I18n.i18n( "PanelSonderfertigkeiten.Lernmethode" );

    /**
     * Die Liste der Sonderfertigkeiten die der Held besitzt, oder besitzen
     * sollte.
     */
    private final PanelSonderfertigkeitenListe gewaehlte;

    /* * Die Liste der Leitsonderfertigkeiten der elfischen Helden. */
    // private final PanelSonderfertigkeitenListe leitsonderfertigkeiten;

    /** Die Liste von Panel. */
    private final List< PanelSonderfertigkeitenListe > panel;

    /** Die Liste mit den verbilligten Sonderfertigkeiten. */
    private final PanelSonderfertigkeitenListe verbilligte;

    /**
     * Erzeugt das Register mit den einzelnen Listen.
     *
     * @param zumSteigern {@code true}, wenn das Panel beim Steigern eines
     *                    Helden verwendet wird, {@code false}, wenn das Panel
     *                    bei der Heldenerschaffung verwendet wird.
     */
    public PanelSonderfertigkeiten( final boolean zumSteigern ) {

        final Vector< Lernmethode > lern = new Vector< Lernmethode >();
        lern.add( Lernmethode.LEHRMEISTER );
        lern.add( Lernmethode.SPEZIELLE_ERFAHRUNG );
        lern.add( Lernmethode.KEINE );
        final JComboBox lernmethode = new JComboBox( lern );

        final Lernmethoden lernmethoden = new Lernmethoden( lernmethode );

        gewaehlte = new PanelSonderfertigkeitenListe(
                true, zumSteigern, lernmethoden );
/*
        leitsonderfertigkeiten = new PanelSonderfertigkeitenListe(
                false, zumSteigern, lernmethoden );
*/
        verbilligte = new PanelSonderfertigkeitenListe(
                false, zumSteigern, lernmethoden );

        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );

        tabbedPane.addTab(
                I18n.i18n( "PanelSonderfertigkeiten.Reiter.Gewaehlte" ),
                erzeugeFrame(
                        I18n.i18n( "PanelSonderfertigkeiten.Gewaehlte" ),
                        gewaehlte ) );

        tabbedPane.addTab(
                I18n.i18n( "PanelSonderfertigkeiten.Reiter.verbilligte" ),
                erzeugeFrame(
                        I18n.i18n( "PanelSonderfertigkeiten.verbilligte" ),
                        verbilligte ) );
/*
        tabbedPane.addTab(
                I18n.i18n( "PanelSonderfertigkeiten.Reiter." +
                		"Leitsonderfertigkeiten" ),
                erzeugeFrame(
                        I18n.i18n( "PanelSonderfertigkeiten." +
                        		"Leitsonderfertigkeiten" ),
                        leitsonderfertigkeiten ) );
*/
        final List< Sonderfertigkeitgruppe > gruppen =
            Sonderfertigkeitgruppe.getGruppen();

        panel = new ArrayList< PanelSonderfertigkeitenListe >( gruppen.size() );

        for ( final Sonderfertigkeitgruppe gruppe : gruppen ) {

            final List< SonderfertigkeitWert > werte = erzeugeWerte( gruppe );

            final PanelSonderfertigkeitenListe p =
                new PanelSonderfertigkeitenListe(
                        false, zumSteigern, lernmethoden );

            p.setWerte( werte );

            panel.add( p );

            tabbedPane.addTab(
                    gruppe.getKuerzel(),
                    erzeugeFrame( gruppe.getName(), p ) );

        }

        if ( zumSteigern  ) {

            setLayout( new FormLayout( "P:G, P, 2dlu, P, P:G", "2dlu, P, 2dlu, F:P:G" ) );

            final CellConstraints cc = new CellConstraints();

            add( new JLabel( LERNMETHODE ), cc.xy ( 2, 2 ) );

            add( lernmethode, cc.xy( 4, 2 ) );

            add( tabbedPane, cc.xyw( 1, 4, 5 ) );

        } else {

            setLayout( new BorderLayout() );

            add( tabbedPane, BorderLayout.CENTER );
        }
    }

    /**
     * Packt ein Panel in einen {@link SimpleInternalFrame} und gibt ihm eine
     * Ueberschrift.
     *
     * @param titel Die Ueberschrift des Frames.
     * @param inhalt des Frames.
     * @return der Frame mit angegebenem Inhalt und Titel.
     */
    private final SimpleInternalFrame erzeugeFrame(
            final String titel,
            final JPanel inhalt ) {

        final SimpleInternalFrame frame;
        frame = new SimpleInternalFrame( "<html><b>" + titel );

        frame.setContent( inhalt );

        return frame;
    }

    /**
     * Erzeugt eine Liste von Werten zu einer Sonderfertigkeit-Gruppe.
     *
     * @param gruppe von Sonderfertigkeiten.
     * @return eine Liste von Werten.
     */
    private final List< SonderfertigkeitWert > erzeugeWerte(
            final List< Sonderfertigkeit > gruppe ) {

        final List< SonderfertigkeitWert > werte =
            new ArrayList< SonderfertigkeitWert >();
/*
        if ( 1 == gruppe.size() && gruppe.get( 0 ).hatVarianten() ) {

            // Eine Liste mit allen Varianten der Sonderfertigkeit bilden.
            // Fuer Kulturkunde.
            final Sonderfertigkeit sonderfertigkeit = gruppe.get( 0 );

            for ( final Object variante :
                                sonderfertigkeit.getVarianten( null ) ) {

                werte.add( new SonderfertigkeitWert( sonderfertigkeit,
                                                     variante,
                                                     false ) );
            }

        } else {
*/
            // eine Liste aus allen Sonderfertigkeiten bilden.

            for ( final Sonderfertigkeit sonderfertigkeit : gruppe ) {

                if ( sonderfertigkeit.hatVarianten() ) {

                    // die Variante bestimmten:
                    final Object variante;

                    final List< Object > varianten =
                        sonderfertigkeit.getVarianten( null );

                    if ( 0 == varianten.size() ) {

                        variante = "";

                    } else {

                        variante = varianten.get( 0 );
                    }

                    werte.add( new SonderfertigkeitWert( sonderfertigkeit,
                                                         variante ) );

                } else {

                    werte.add( new SonderfertigkeitWert( sonderfertigkeit ) );

                }
            }
  //      }

        return werte;
    }

    /**
     * Muss zum Steigern eines Helden aufgerufen werden. Bestimmt den Helden
     * der ueber das Panel editiert wird.
     *
     * @param derHeld
     *            der im Panel bearbeitet wird.
     */
    public final void setHeld( final Held held ) {

        gewaehlte.setHeld( held );

        // leitsonderfertigkeiten.setWerte( held.getLeitsonderfertigkeiten() );
        // leitsonderfertigkeiten.setHeld( held );

        final List< SonderfertigkeitWert > verbilligteWerte =
            erzeugeWerte( held.getVerbilligteSonderfertigkeiten() );
        verbilligteWerte.addAll( held.getVerbilligteVarianten() );

        verbilligte.setWerte( verbilligteWerte );

        verbilligte.setHeld( held );

        for ( final PanelSonderfertigkeitenListe p : panel ) {

            p.setHeld( held );
        }

    }

    /**
     * Muss bei der Heldenerschaffung aufgerufen werden.
     * Bestimmt den Manager fuer die aktuell Erschaffung und damit
     * auch den Helden dessen Werte ueber die Schnittstelle manipuliert
     * werden.
     *
     * @param manager fuer die aktulle Erschaffung.
     */
    public final void setManager( final ErschaffungManager manager ) {

        gewaehlte.setManager( manager );

        final Held held = manager.getHeld();

        // leitsonderfertigkeiten.setWerte( held.getLeitsonderfertigkeiten() );
        // leitsonderfertigkeiten.setManager( manager );

        for ( final PanelSonderfertigkeitenListe p : panel ) {

            p.setManager( manager );
        }

        final List< SonderfertigkeitWert > verbilligteWerte =
            erzeugeWerte( held.getVerbilligteSonderfertigkeiten() );

        verbilligteWerte.addAll( held.getVerbilligteVarianten() );

        verbilligte.setWerte( verbilligteWerte );

        verbilligte.setManager( manager );

    }
}
