/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JPanel;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.factories.DefaultComponentFactory;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

class PanelMetaLeittalente extends JPanel {

    /**
     * Die Checkbox an Position i steht fuer das Talent, das in {@link #talente}
     * an Position i steht.
     */
    private final List< JCheckBox > checkBoxen;

    /** Die Talente aus denen man die Leittalente waehlen kann. */
    private final List< Talent > talente;

    PanelMetaLeittalente() {

        checkBoxen = new ArrayList< JCheckBox >();

        talente = new ArrayList< Talent >();

        final FormLayout layout = new FormLayout( "F:P:G" );

        final DefaultFormBuilder builder =
            new DefaultFormBuilder( layout, this );

        builder.setDefaultDialogBorder();

        final DefaultComponentFactory factory =
            DefaultComponentFactory.getInstance();

        for( final Talentgruppe gruppe : Talentgruppe.getTalentgruppen() ) {

            builder.append( factory.createSeparator( gruppe.getName() ) );

            for ( final Talent talent : gruppe ) {

                talente.add( talent );

                final JCheckBox checkBox = new JCheckBox( talent.getName() );

                checkBoxen.add( checkBox );

                builder.append( checkBox );
            }
        }
    }

    /**
     * Setzt die im Panel gewaehlten Talente auf die Leittalente des Bausteins.
     *
     * @param baustein
     *            dessen Leittalente im Panel angzeigt werden sollen.
     */
    public final void anzeigen( final Heldenbaustein baustein ) {

        final List< Talent > leittalente = baustein.getLeittalente();

        for ( int i = 0, n = talente.size() ; i < n ; ++i ) {

            final boolean gewaehlt = leittalente.contains( talente.get( i ) );

            checkBoxen.get( i ).setSelected( gewaehlt );
        }
    }

    /** Waehlt alle Talente im Panel ab. */
    public void reset() {
        for ( final JCheckBox checkBox : checkBoxen ) {

            checkBox.setSelected( false );
        }
    }

    /**
     * Setzt die Leittalente des Bausteins, auf die im Panel gewaehlten.
     *
     * @param baustein
     *            der die Leittalente erhalten soll.
     */
    public final void uebertrageDaten( final Heldenbaustein baustein ) {

        final List< Talent > leittalente = new ArrayList< Talent >();

        for ( int i = 0, n = talente.size(); i < n ; ++i ) {

            if ( checkBoxen.get( i ).isSelected() ) {

                leittalente.add( talente.get( i ) );
            }
        }

        baustein.setLeittalente( leittalente );
    }

}
