/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Vector;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.ungeeignete.UngeeigneterVorNachteil;
import de.bernhardjung.dsaprogramm.bausteine.ungeeignete.UngeeigneterVorNachteilAusser;
import de.bernhardjung.dsaprogramm.bausteine.ungeeignete.UngeeigneterVorNachteilStandard;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

public class PanelMetaUngeeigneteVorNachteile extends JPanel {

    private static final String AUSSER =
        I18n.i18n( "PanelMetaUngeeigneteVorNachteile.Ausser" );

    private final List< VorNachteil > vorNachteile;

    private final List< JCheckBox > gewaehlt;

    private final List< JCheckBox > ausserCheckBoxen;

    private final List< JComboBox > ausserComboBoxen;

    PanelMetaUngeeigneteVorNachteile() {

        vorNachteile = VorNachteil.getVorNachteile();

        gewaehlt = new ArrayList< JCheckBox >( vorNachteile.size() );

        ausserCheckBoxen = new ArrayList< JCheckBox >( vorNachteile.size() );

        ausserComboBoxen = new ArrayList< JComboBox >( vorNachteile.size() );

        final Vector< VorNachteil > vorNachteileVector =
            new Vector< VorNachteil >( vorNachteile );

        for ( final VorNachteil vorNachteil : vorNachteile ) {

            gewaehlt.add( new JCheckBox( vorNachteil.getName() ) );

            ausserCheckBoxen.add( new JCheckBox( AUSSER ) );

            ausserComboBoxen.add( new JComboBox( vorNachteileVector ) );
        }

        final FormLayout layout = new FormLayout( "F:160px:G, 4dlu, P, 4dlu, F:160px:G" );

        final DefaultFormBuilder builder =
            new DefaultFormBuilder( layout, this );

        builder.setDefaultDialogBorder();

        for ( int i = 0, n = vorNachteile.size() ; i < n ; ++i ) {

            builder.append( gewaehlt.get( i ) );

            builder.append( ausserCheckBoxen.get( i ) );

            builder.append( ausserComboBoxen.get( i ) );
        }
    }

    public final void anzeigen(
            final Collection< UngeeigneterVorNachteil > ungeeignete ) {

        for ( int i = vorNachteile.size() - 1 ; i >= 0 ; --i ) {

            gewaehlt.get( i ).setSelected( false );

            ausserCheckBoxen.get( i ).setSelected( false );
        }

        for ( final UngeeigneterVorNachteil ungeeigneter : ungeeignete ) {

            if( ungeeigneter instanceof UngeeigneterVorNachteilAusser ) {

                final VorNachteil vorNachteil = ungeeigneter.getVorNachteil();

                final VorNachteil ausser =
                    ((UngeeigneterVorNachteilAusser)ungeeigneter).getAusser();

                final int index = Collections.binarySearch( vorNachteile,
                                                            vorNachteil   );

                gewaehlt.get( index ).setSelected( true );

                ausserCheckBoxen.get( index ).setSelected( true );

                ausserComboBoxen.get( index ).setSelectedItem( ausser );

            } else {

                final VorNachteil vorNachteil = ungeeigneter.getVorNachteil();

                final int index = Collections.binarySearch( vorNachteile,
                                                            vorNachteil   );

                gewaehlt.get( index ).setSelected( true );
            }
        }
    }

    public final Collection< UngeeigneterVorNachteil > getGewaehlteVorNachteile() {

        final Collection< UngeeigneterVorNachteil > ungeeignete =
            new ArrayList< UngeeigneterVorNachteil >();

        for ( int i = vorNachteile.size() - 1 ; i >= 0 ; --i ) {

            if ( gewaehlt.get( i ).isSelected() ) {

                if ( ausserCheckBoxen.get( i ).isSelected() ) {

                    final VorNachteil vorNachteil = vorNachteile.get( i );

                    final VorNachteil ausser = (VorNachteil)
                        ausserComboBoxen.get( i ).getSelectedItem();

                    ungeeignete.add( new UngeeigneterVorNachteilAusser(
                            vorNachteil, ausser ) );
                } else {

                    final VorNachteil vorNachteil = vorNachteile.get( i );

                    ungeeignete.add( new UngeeigneterVorNachteilStandard(
                            vorNachteil ) );

                }
            }
        }

        return ungeeignete;
    }

    /** Setzt die Auswahl im Panel zurueck. */
    public void reset() {

        for ( int i = 0, n = gewaehlt.size(); i < n ; ++i ) {

            gewaehlt.get( i ).setSelected( false );

            ausserCheckBoxen.get( i ).setSelected(  false );
        }
    }
}
