/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;

/**
 * Eine Voraussetzung bei der ein Held eine andere Sonderfertigkeit besitzen
 * muss.
 * 
 * @author Bernhard Jung
 */
class VoraussetzungSonderfertigkeit implements Voraussetzung {

    /** Eine Beschreibung der Voraussetzung fuer den Benutzer. */
    private final String beschreibungString;
    
    /** Die Sonderfertigkeit die voraussgesetzt wird. */
    private final Sonderfertigkeit sonderfertigkeit;
    
    /**
     * @param dieSonderfertigkeit die vorausgesetzt wird.
     */
    VoraussetzungSonderfertigkeit( 
            final Sonderfertigkeit dieSonderfertigkeit ) {
        
        sonderfertigkeit = dieSonderfertigkeit;
        
        beschreibungString = String.format(
                I18n.i18n( "VoraussetzungSonderfertigkeit" ),
                sonderfertigkeit.getName() );
    }
    
    /** @return die Sonderfertigkeit */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( (Object) sonderfertigkeit );
    }
    
    /**
     * Prueft ob der Held die Sonderfertigkeit besitzt.
     */
    public KonfliktBeschreibung pruefen(
            final Held held, 
            final SonderfertigkeitWert wert ) {
        
        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        if ( ! held.hat( sonderfertigkeit ) ) {
            
            beschreibung.einfuegen( beschreibungString );
        }
        
        return beschreibung;
    }

}
