/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.voraussetzungen;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;

/**
 * Die Standard-Voraussetzung fuer jeden Sonderfertigkeit.
 * Sie prueft ob der Held eine Sonderfertigkeit, die ihm durch Rasse, Kultur
 * oder Profession vorgegeben ist, auch besitzt.
 *
 * @author Bernhard Jung
 */
class VoraussetzungStandard implements Voraussetzung {

    private static final String VORGEGEBEN =
        I18n.i18n( "VoraussetzungStandard.Sf.Vorgegeben" );

    /**
     * @return eine leere Liste.
     */
    @SuppressWarnings( "unchecked" )
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    public KonfliktBeschreibung pruefen(
            final Held held,
            final SonderfertigkeitWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if ( held.hatVorgegeben( wert )
                && ( ! held.hat( wert ) ) ) {

            beschreibung.einfuegen( VORGEGEBEN );
        }

        return beschreibung;
    }

}
