/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.anpasser;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.eigenschaften.EigenschaftWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

/**
 * Setzt eine Eigenschaft auf eine bestimmte Mindeststufe.
 *
 * @author Bernhard Jung
 *
 */
class AnpasserEigenschaft implements Anpasser {

    /** Die Eigenschaft deren Stufe angepasst werden soll. */
    private final Eigenschaft eigenschaft;

    /** Die Stufe die die Eigenschaft mindestens besitzen soll. */
    private final int stufe;

    /**
     * @param dieEigenschaft
     *            deren Stufe angepasst werden soll.
     * @param dieStufe
     *            die die Eigenschaft mindestens besitzen soll.
     */
    AnpasserEigenschaft(
            final Eigenschaft dieEigenschaft,
            final int dieStufe ) {

        eigenschaft = dieEigenschaft;

        stufe = dieStufe;
    }

    /**
     * Setzt die Eigenschaft des Helden auf die vorgegebene Stufe, falls ihre
     * Stufe aktuell kleiner ist, als die vorgegebene Stufe.
     */
    public void eingefuegt( final Held held, final VorNachteilWert wert ) {

        final EigenschaftWert eigenschaftWert = held.getWert( eigenschaft );

        final int modifikation = eigenschaftWert.getModifikation();

        if ( eigenschaftWert.getStufe() < stufe ) {

            eigenschaftWert.setBasisstufe( stufe - modifikation );

        }
    }

    /** Tut nichts. */
    public void entfernt(final Held held, final VorNachteilWert wert) {
        // ignorieren
    }

    /** Macht nichts. */
    public void stufeGeaendert(final Held held, final VorNachteilWert wert) {
        // nichts zu tun.
    }

    public void geaendertVariante(
            final Held held,
            final Object alteVariante,
            final Object neueVariante ) {
        // nichts zu tun.
    }

}
