/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.kosten;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

/**
 * Die Kosten werden um die Kosten einer Sonderfertigkeit verringert,
 * wenn der Held diese durch Kultur oder Profession erhalten hat.
 *
 * @author Bernhard Jung
 */
class KostenSonderfertigkeit implements Kosten {

    /** Die GP-Kosten die normalerweise zu zahlen sind. */
    private final int kosten;

    /** Die Sonderfertigkeit deren Kosten angerechnet werden. */
    private final SonderfertigkeitWert sonderfertigkeit;

    KostenSonderfertigkeit(
            final int dieKosten,
            final Sonderfertigkeit dieSonderfertigkeit ) {

        kosten = dieKosten;

        sonderfertigkeit = new SonderfertigkeitWert( dieSonderfertigkeit );
    }

    /** @return 0 */
    public int getAbbaukosten(final Held held, final VorNachteilWert wert) {
        return 0;
    }

    /** @return eine leere Liste. */
    @SuppressWarnings( "unchecked" )
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /** {@inheritDoc} */
    public int getGpKosten() {
        return kosten;
    }

    /** {@inheritDoc} */
    public int getGpKosten(final VorNachteilWert wert) {
        return kosten;
    }

    /** {@inheritDoc} */
    public int getGpKosten(
            final Held held,
            final VorNachteilWert wert ) {

        if ( held.hatVorgegeben( wert ) ) {

            return 0;
        }

        if ( held.hatVorgegeben( sonderfertigkeit ) ) {

            return ( kosten - sonderfertigkeit.getGpKosten() );

        } else {

            return kosten;

        }
    }

    /** @return 0 */
    public int getSenkungskosten(final Lernmethode lernmethode) {
        return 0;
    }

    /** @return {@code false} */
    public final boolean istSenkbar() {
        return false;
    }
}
