/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Augenfarbe;
import de.bernhardjung.dsaprogramm.basis.Haarfarbe;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class VoraussetzungAlbino extends Voraussetzung {

    /** Erklaert, dass der Charakter rote oder violett Augen haben muss. */
    private static final String AUGENFARBE =
        I18n.i18n( "VoraussetzungAlbino.Augenfarbe" );

    /** Erklaert, dass der Charakter weisse Haare haben muss. */
    private static final String HAARFARBE =
        I18n.i18n( "VoraussetzungAlbino.Haarfarbe" );

    /** @return {@link Held#HAARFARBE} und {@link Held#AUGENFARBE}. */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( Held.AUGENFARBE, Held.HAARFARBE );
    }

    /**
     * Ueberprueft Haar- und Augenfarbe des Helden.
     */
    public KonfliktBeschreibung pruefen(
            final Held held, 
            final VorNachteilWert wert ) {
        
        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        if ( ! held.getHaarfarbe().equals( Haarfarbe.WEISS ) ) {
            
            beschreibung.einfuegen( HAARFARBE );
        }
        
        final String augenfarbe = held.getAugenfarbe();
        
        if ( ( ! augenfarbe.equals( Augenfarbe.ROT ) )
          && ( ! augenfarbe.equals( Augenfarbe.VIOLETT ) ) ) {
            
            beschreibung.einfuegen( AUGENFARBE );
        }
        
        return beschreibung;
    }

}
