/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

/**
 * Eine Voraussetzung fuer einen Vor-/Nachteil der nicht mit einer bestimmten
 * Rasse kombiniert werden kann.
 *
 * @author Bernhard Jung
 *
 */
class VoraussetzungNichtRasse extends Voraussetzung {

    /**
     * Erklaert dem Benutzer das die spezielle Rassen den Vor-Nachteil nicht
     * waehlen kann. In den String muss die Rasse noch mit String.format()
     * eingesetzt werden.
     */
    private static final String BESCHREIBUNG =
        I18n.i18n( "VoraussetzungNichtRasse" );

    /**
     * Erklaert dem Benutzer das die spezielle Rassen den Vor-Nachteil nicht
     * waehlen kann.
     */
    private final String beschreibungString;

    /**
     * Die Rasse fuer die der Vor-/Nachteil nicht waehlbar ist.
     */
    private final Rasse rasse;

    /**
     * @param dieId
     *            der Rasse die nicht mit dem Vor-/Nachteil kombiniert werden
     *            kann.
     */
    VoraussetzungNichtRasse( final String dieId ) {
        rasse = Datenspeicher.getInstanz().getRasse( dieId );

        beschreibungString = String.format( BESCHREIBUNG, rasse.getName() );
    }

    /**
     * @return eine leere Liste.
     */
    @Override
    @SuppressWarnings( "unchecked" )
    public Collection< Object > getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /** Prueft ob der Held die Rasse bestizt. */
    @Override
    public KonfliktBeschreibung pruefen( final Held held, final VorNachteilWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if ( held.getRasse().equals( rasse ) ) {

            beschreibung.einfuegen( beschreibungString );
        }

        return beschreibung;
    }

}
