/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.zauber.voraussetzungen;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Strings;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;

/**
 * Voraussetzung fuer Zauber die nur von einer bestimmten Kultur
 * gewaehlt werden duerfen.
 *
 * TODO wird nicht mehr benoetigt.
 *
 * @author Bernhard Jung
 *
 */
class VoraussetzungKultur implements Voraussetzung {

    /** "nur fuer Kultur %1$s" */
    private static final String NUR =
        I18n.i18n( "VoraussetzungKultur" );

    /** Die vorausgesetzten Kulturen. */
    private final List< Kultur > kulturen;

    /**
     * @param dieIds der geforderten Kulturen.
     */
    VoraussetzungKultur( final String ... dieIds ) {

        kulturen = new ArrayList< Kultur >( dieIds.length );

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final String id : dieIds ) {

            kulturen.add( speicher.getKultur( id ) );
        }

    }

    /** @return eine leere Collection */
    @SuppressWarnings( "unchecked" )
    public final Collection< Object > getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /** {@inheritDoc} */
    public KonfliktBeschreibung pruefen( final Held held, final ZauberWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        boolean erfuelltVoraussetzung = false;

        for ( final Kultur kultur : kulturen ) {

            if ( held.getKultur().equals( kultur ) ) {

                erfuelltVoraussetzung = true;

                break;
            }
        }

        if ( ! erfuelltVoraussetzung ) {

            beschreibung.einfuegen( String.format( NUR,
                    Strings.erzeugeAufzaehlungMitUnd( kulturen ) ) );
        }

        return beschreibung;
    }

}
